@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,13 +12,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #ifndef MIR_GRAPHICS_DISPLAY_H_
 #define MIR_GRAPHICS_DISPLAY_H_
 
+#include "mir/graphics/frame.h"
+#include "mir/renderer/gl/context_source.h"
 #include <memory>
 #include <functional>
 #include <chrono>
@@ -32,8 +32,6 @@ namespace graphics
 class DisplayBuffer;
 class DisplayConfiguration;
 class Cursor;
-class CursorImage;
-class GLContext;
 class EventHandlerRegister;
 
 typedef std::function<bool()> DisplayPauseHandler;
@@ -71,6 +69,9 @@ public:
      * scene too early results in up to one whole frame of extra lag if
      * rendering is fast or skipped altogether (bypass/overlays). But sampling
      * too late and we might miss the deadline. If unsure just return zero.
+     *
+     * This is equivalent to:
+     * https://www.opengl.org/registry/specs/NV/glx_delay_before_swap.txt
      */
     virtual std::chrono::milliseconds recommended_sleep() const = 0;
 
@@ -84,7 +85,7 @@ protected:
 /**
  * Interface to the display subsystem.
  */
-class Display
+class Display : public renderer::gl::ContextSource
 {
 public:
     /**
@@ -97,8 +98,31 @@ public:
      */
     virtual std::unique_ptr<DisplayConfiguration> configuration() const = 0;
 
+    /**
+     * Applying a display configuration only if it will not invalidate existing DisplayBuffers
+     *
+     * The Display must guarantee that the references to the DisplayBuffer acquired via
+     * DisplaySyncGroup::for_each_display_buffer() remain valid until the Display is destroyed or
+     * Display::configure() is called.
+     *
+     * If this function returns \c true then the new display configuration has been applied.
+     * If this function returns \c false then the new display configuration has not been applied.
+     *
+     * In either case this function guarantees that existing DisplayBuffer references will remain
+     * valid.
+     *
+     * \param conf [in] Configuration to possibly apply.
+     * \return      \c true if \p conf has been applied as the new output configuration.
+     */
+    virtual bool apply_if_configuration_preserves_display_buffers(DisplayConfiguration const& conf) = 0;
+
     /**
      * Sets a new output configuration.
+     *
+     * \note   A call to configure() may invalidate any and all existing DisplayBuffers. See
+     *         \ref apply_if_configuration_preserves_display_buffers for a configure that guarantees
+     *         preservation of all DisplayBuffers.
+     *         DisplayBuffers may \em only be invalidated by a call to configure.
      */
     virtual void configure(DisplayConfiguration const& conf) = 0;
 
@@ -142,19 +166,26 @@ public:
     /**
      * Create a hardware cursor object.
      */
-    virtual std::shared_ptr<Cursor> create_hardware_cursor(std::shared_ptr<CursorImage> const& initial_image) = 0;
+    virtual std::shared_ptr<Cursor> create_hardware_cursor() = 0;
 
     /**
-     * Creates a GLContext object that shares resources with the Display's GL context.
+     * Returns timing information for the last frame displayed on a given
+     * output.
      *
-     * This is usually implemented as a shared EGL context. This object can be used
-     * to access graphics resources from an arbitrary thread.
+     * Frame timing will be provided to clients only when they request it.
+     * This is to ensure idle clients never get woken by unwanted events.
+     * It is also distinctly separate from the display configuration as this
+     * timing information changes many times per second and should not interfere
+     * with the more static display configuration.
+     *
+     * Note: Using unsigned here because DisplayConfigurationOutputId is
+     * troublesome (can't be forward declared) and including
+     * display_configuration.h to get it would be an overkill.
      */
-    virtual std::unique_ptr<GLContext> create_gl_context() = 0;
+    virtual Frame last_frame_on(unsigned output_id) const = 0;
 
-protected:
     Display() = default;
-    virtual ~Display() {/* TODO: make nothrow */}
+    virtual ~Display() = default;
 private:
     Display(Display const&) = delete;
     Display& operator=(Display const&) = delete;
