@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Christopher James Halse Rogers <christopher.halse.rogers@canonical.com>
  */
 
 #ifndef MIR_PLATFORM_MODULE_PROPERTIES_H_
@@ -23,7 +21,11 @@ namespace mir
 {
 
 /**
- * Describes a platform module
+ * Describes a platform module. Mir provides graphics platforms named "mir:*"
+ * Mir provides a "mir:evdev-input" input platform.
+ *
+ * Third party platforms should be named according to the vendor and platform:
+ *  "<vendor>:<platform>"
  */
 struct ModuleProperties
 {
@@ -31,6 +33,7 @@ struct ModuleProperties
     int major_version;
     int minor_version;
     int micro_version;
+    char const* file;
 };
 }
 
