@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Christopher James Halse Rogers <christopher.halse.rogers@canonical.com>
  */
 
 #include "mir/shared_library_prober.h"
@@ -23,6 +21,7 @@
 
 #include <system_error>
 #include <cstring>
+#include <algorithm>
 
 namespace
 {
@@ -63,8 +62,10 @@ bool greater_soname_version(boost::filesystem::path const& lhs, boost::filesyste
 }
 }
 
-std::vector<std::shared_ptr<mir::SharedLibrary>>
-mir::libraries_for_path(std::string const& path, mir::SharedLibraryProberReport& report)
+void mir::select_libraries_for_path(
+    std::string const& path,
+    std::function<Selection(std::shared_ptr<mir::SharedLibrary> const&)> const& selector,
+    mir::SharedLibraryProberReport& report)
 {
     report.probing_path(path);
     // We use the error_code overload because we want to throw a std::system_error
@@ -87,20 +88,33 @@ mir::libraries_for_path(std::string const& path, mir::SharedLibraryProberReport&
 
     std::sort(libraries.begin(), libraries.end(), &greater_soname_version);
 
-    std::vector<std::shared_ptr<mir::SharedLibrary>> result;
-
     for(auto& lib : libraries)
     {
         try
         {
             report.loading_library(lib);
-            result.emplace_back(std::make_shared<mir::SharedLibrary>(lib.string()));
+            auto const shared_lib = std::make_shared<mir::SharedLibrary>(lib.string());
+
+            if (selector(shared_lib) == Selection::quit)
+                return;
         }
         catch (std::runtime_error const& err)
         {
             report.loading_failed(lib, err);
         }
     }
+}
+
+std::vector<std::shared_ptr<mir::SharedLibrary>>
+mir::libraries_for_path(std::string const& path, mir::SharedLibraryProberReport& report)
+{
+    std::vector<std::shared_ptr<mir::SharedLibrary>> result;
+
+    select_libraries_for_path(
+        path,
+        [&](std::shared_ptr<mir::SharedLibrary> const& shared_lib)
+            { result.push_back(shared_lib); return Selection::persist; },
+        report);
 
     return result;
 }
