@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,17 +12,11 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
  */
 
 #ifndef MIR_LOGGING_INPUT_TIMESTAMP_H_
 #define MIR_LOGGING_INPUT_TIMESTAMP_H_
 
-#define MIR_INCLUDE_DEPRECATED_EVENT_HEADER
-
-#include "mir_toolkit/event.h"
-
 #include <string>
 #include <chrono>
 
