@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alberto Aguirre <alberto.aguirre@canonical.com>
  */
 
 #ifndef MIR_LOCKABLE_CALLBACK_WRAPPER_H_
@@ -29,10 +27,10 @@ namespace mir
 class LockableCallbackWrapper : public LockableCallback
 {
 public:
-    LockableCallbackWrapper(std::shared_ptr<LockableCallback> const& wrapped,
+    LockableCallbackWrapper(std::unique_ptr<LockableCallback> wrapped,
                             std::function<void()> const& precall_hook);
 
-    LockableCallbackWrapper(std::shared_ptr<LockableCallback> const& wrapped,
+    LockableCallbackWrapper(std::unique_ptr<LockableCallback> wrapped,
                             std::function<void()> const& precall_hook,
                             std::function<void()> const& postcall_hook);
 
@@ -41,7 +39,7 @@ public:
     void unlock() override;
 
 private:
-    std::shared_ptr<LockableCallback> wrapped_callback;
+    std::unique_ptr<LockableCallback> wrapped_callback;
     std::function<void()> precall_hook;
     std::function<void()> postcall_hook;
 };
