@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #ifndef MIR_SCENE_SURFACE_EVENT_SOURCE_H_
@@ -38,20 +36,20 @@ class SurfaceEventSource : public NullSurfaceObserver
 public:
     SurfaceEventSource(
         frontend::SurfaceId id,
-        Surface const& surface,
         OutputPropertiesCache const& outputs,
         std::shared_ptr<frontend::EventSink> const& event_sink);
 
-    void attrib_changed(MirSurfaceAttrib attrib, int value) override;
-    void resized_to(geometry::Size const& size) override;
-    void moved_to(geometry::Point const& top_left) override;
-    void orientation_set_to(MirOrientation orientation) override;
-    void client_surface_close_requested() override;
-    void keymap_changed(xkb_rule_names const& names) override;
+    void attrib_changed(Surface const* surf, MirWindowAttrib attrib, int value) override;
+    void content_resized_to(Surface const* surf, geometry::Size const& content_size) override;
+    void moved_to(Surface const* surf, geometry::Point const& top_left) override;
+    void orientation_set_to(Surface const* surf, MirOrientation orientation) override;
+    void client_surface_close_requested(Surface const* surf) override;
+    void placed_relative(Surface const* surf, geometry::Rectangle const& placement) override;
+    void input_consumed(Surface const* surf, std::shared_ptr<MirEvent const> const& event) override;
+    void start_drag_and_drop(Surface const* surf, std::vector<uint8_t> const& handle) override;
 
 private:
     frontend::SurfaceId const id;
-    Surface const& surface;
     OutputPropertiesCache const& outputs;
     std::weak_ptr<OutputProperties const> last_output;
     std::shared_ptr<frontend::EventSink> const event_sink;
