@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "mir/graphics/buffer_basic.h"
@@ -27,12 +25,7 @@ mg::BufferID generate_next_buffer_id()
 {
     static std::atomic<uint32_t> next_id{0};
 
-    auto id = mg::BufferID(next_id.fetch_add(1));
-
-    // Avoid returning an "invalid" id. (Not sure we need invalid ids)
-    while (!id.is_valid()) id = mg::BufferID(next_id.fetch_add(1));
-
-    return id;
+    return mg::BufferID(next_id.fetch_add(1));
 }
 }
 
