@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,16 +12,14 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "mir/options/program_option.h"
+#include "mir/log.h"
 
 #include <boost/program_options/parsers.hpp>
 
 #include <fstream>
-#include <iostream>
 
 namespace mo = mir::options;
 namespace po = boost::program_options;
@@ -43,7 +41,10 @@ void mo::ProgramOption::parse_arguments(
     int argc,
     char const* argv[])
 {
-    auto parsed_command_line = po::command_line_parser(argc, argv).options(desc).allow_unregistered().run();
+    // Passing argc and argv directly to command_line_parser() should do same thing. Unfortunately, when argc is 0, boost
+    // 1.67+ does not correctly construct the vector and throws a std::bad_alloc
+    std::vector<std::string> args{argv + 1, argv + std::max(argc, 1)};
+    auto parsed_command_line = po::command_line_parser(args).options(desc).allow_unregistered().run();
     po::store(parsed_command_line, options);
     po::notify(options);
 
@@ -72,7 +73,7 @@ void mo::ProgramOption::parse_environment(
              for(auto& ch : result)
              {
                  if (ch == '_') ch = '-';
-                 else ch = tolower(ch);
+                 else ch = std::tolower(ch, std::locale::classic()); // avoid current locale
              }
 
              return result;
@@ -111,7 +112,7 @@ void mo::ProgramOption::parse_file(
         }
         catch (const po::error& error)
         {
-            std::cerr << "ERROR in " << filename << ": " << error.what() << std::endl;
+            log_warning("Error in %s: %s", filename.c_str(), error.what());
         }
     }
 
