@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
 #include "libinput.h"
@@ -21,9 +19,14 @@
 
 namespace mie = mir::input::evdev;
 
-mie::LibInputDevicePtr mie::make_libinput_device(libinput* lib, char const* path)
+void mie::LibInputDeviceDeleter::operator()(::libinput_device* device) const
 {
-    auto ret = mie::LibInputDevicePtr(::libinput_path_add_device(lib, path), libinput_device_unref);
+    libinput_device_unref(device);
+}
+
+mie::LibInputDevicePtr mie::make_libinput_device(std::shared_ptr<libinput> const& lib, libinput_device* dev)
+{
+    auto ret = mie::LibInputDevicePtr(dev, lib);
     if (ret)
         libinput_device_ref(ret.get());
     return ret;
