@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,48 +12,39 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "mir/default_server_configuration.h"
 
+#include "mir/shell/shell.h"
 #include "buffer_stream_factory.h"
 #include "default_display_buffer_compositor_factory.h"
 #include "multi_threaded_compositor.h"
-#include "gl_renderer_factory.h"
-#include "compositing_screencast.h"
-#include "timeout_frame_dropping_policy_factory.h"
+#include "gl/renderer_factory.h"
+#include "basic_screen_shooter.h"
+#include "null_screen_shooter.h"
 #include "mir/main_loop.h"
+#include "mir/graphics/display.h"
+#include "mir/executor.h"
+#include "mir/renderer/gl/basic_buffer_render_target.h"
+#include "mir/renderer/gl/context.h"
+#include "mir/renderer/renderer.h"
+#include "mir/log.h"
 
-#include "mir/frontend/screencast.h"
 #include "mir/options/configuration.h"
 
-#include <boost/throw_exception.hpp>
-
 namespace mc = mir::compositor;
 namespace ms = mir::scene;
 namespace mf = mir::frontend;
+namespace mrg = mir::renderer::gl;
 
 std::shared_ptr<ms::BufferStreamFactory>
 mir::DefaultServerConfiguration::the_buffer_stream_factory()
 {
     return buffer_stream_factory(
-        [this]()
+        []()
         {
-            return std::make_shared<mc::BufferStreamFactory>(the_buffer_allocator(),
-                                                             the_frame_dropping_policy_factory());
-        });
-}
-
-std::shared_ptr<mc::FrameDroppingPolicyFactory>
-mir::DefaultServerConfiguration::the_frame_dropping_policy_factory()
-{
-    return frame_dropping_policy_factory(
-        [this]()
-        {
-            return std::make_shared<mc::TimeoutFrameDroppingPolicyFactory>(the_main_loop(),
-                                                                           std::chrono::milliseconds{100});
+            return std::make_shared<mc::BufferStreamFactory>();
         });
 }
 
@@ -81,34 +72,53 @@ mir::DefaultServerConfiguration::the_compositor()
     return compositor(
         [this]()
         {
+            std::chrono::milliseconds const composite_delay(
+                the_options()->get<int>(options::composite_delay_opt));
+
             return std::make_shared<mc::MultiThreadedCompositor>(
                 the_display(),
                 the_scene(),
                 the_display_buffer_compositor_factory(),
+                the_shell(),
                 the_compositor_report(),
-                !the_options()->is_set(options::host_socket_opt));
+                composite_delay,
+                true);
         });
 }
 
-std::shared_ptr<mc::RendererFactory> mir::DefaultServerConfiguration::the_renderer_factory()
+std::shared_ptr<mir::renderer::RendererFactory> mir::DefaultServerConfiguration::the_renderer_factory()
 {
     return renderer_factory(
-        [this]()
+        []()
         {
-            return std::make_shared<mc::GLRendererFactory>();
+            return std::make_shared<mir::renderer::gl::RendererFactory>();
         });
 }
 
-std::shared_ptr<mf::Screencast> mir::DefaultServerConfiguration::the_screencast()
+auto mir::DefaultServerConfiguration::the_screen_shooter() -> std::shared_ptr<compositor::ScreenShooter>
 {
-    return screencast(
-        [this]()
+    return screen_shooter(
+        [this]() -> std::shared_ptr<compositor::ScreenShooter>
         {
-            return std::make_shared<mc::CompositingScreencast>(
-                the_scene(),
-                the_display(),
-                the_buffer_allocator(),
-                the_display_buffer_compositor_factory()
-                );
+            try
+            {
+                auto render_target = std::make_unique<mrg::BasicBufferRenderTarget>(the_display()->create_gl_context());
+                auto renderer = the_renderer_factory()->create_renderer_for(*render_target);
+                return std::make_shared<compositor::BasicScreenShooter>(
+                    the_scene(),
+                    the_clock(),
+                    thread_pool_executor,
+                    std::move(render_target),
+                    std::move(renderer));
+            }
+            catch (...)
+            {
+                mir::log(
+                    ::mir::logging::Severity::error,
+                    "",
+                    std::current_exception(),
+                    "failed to create BasicScreenShooter");
+                return std::make_shared<compositor::NullScreenShooter>(thread_pool_executor);
+            }
         });
 }
