@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #ifndef MIR_COMPOSITOR_DEFAULT_DISPLAY_BUFFER_COMPOSITOR_FACTORY_H_
@@ -24,24 +22,25 @@
 
 namespace mir
 {
+namespace renderer
+{
+class RendererFactory;
+}
 ///  Compositing. Combining renderables into a display image.
 namespace compositor
 {
-class RendererFactory;
 
-// TODO this is ideally an implementation class. It is only in a public header
-// TODO because it is used in some example code (which probably needs rethinking).
 class DefaultDisplayBufferCompositorFactory : public DisplayBufferCompositorFactory
 {
 public:
     DefaultDisplayBufferCompositorFactory(
-        std::shared_ptr<RendererFactory> const& renderer_factory,
+        std::shared_ptr<renderer::RendererFactory> const& renderer_factory,
         std::shared_ptr<CompositorReport> const& report);
 
     std::unique_ptr<DisplayBufferCompositor> create_compositor_for(graphics::DisplayBuffer& display_buffer);
 
 private:
-    std::shared_ptr<RendererFactory> const renderer_factory;
+    std::shared_ptr<renderer::RendererFactory> const renderer_factory;
     std::shared_ptr<CompositorReport> const report;
 };
 
