@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
  */
 
 #ifndef MIR_COMPOSITOR_QUEUEING_SCHEDULE_H_
@@ -31,9 +29,9 @@ namespace compositor
 class QueueingSchedule : public Schedule
 {
 public:
-    void schedule(std::shared_ptr<graphics::Buffer> const& buffer);
-    bool anything_scheduled();
-    std::shared_ptr<graphics::Buffer> next_buffer();
+    void schedule(std::shared_ptr<graphics::Buffer> const& buffer) override;
+    unsigned int num_scheduled() override;
+    std::shared_ptr<graphics::Buffer> next_buffer() override;
 
 private:
     std::mutex mutable mutex;
