@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,11 +12,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
-#include "default_display_configuration_policy.h"
+#include "mir/graphics/default_display_configuration_policy.h"
 #include "mir/graphics/display_configuration.h"
 #include "mir/graphics/pixel_format_utils.h"
 
@@ -64,22 +62,14 @@ MirPixelFormat select_opaque_format(MirPixelFormat format, std::vector<MirPixelF
 
 }
 
-void mg::DefaultDisplayConfigurationPolicy::apply_to(DisplayConfiguration& conf)
+void mg::CloneDisplayConfigurationPolicy::apply_to(DisplayConfiguration& conf)
 {
     static MirPowerMode const default_power_state = mir_power_mode_on;
-    std::unordered_map<DisplayConfigurationCardId, size_t> available_outputs_for_card;
-
-    conf.for_each_card(
-        [&](DisplayConfigurationCard const& card)
-        {
-            available_outputs_for_card[card.id] = card.max_simultaneous_outputs;
-        });
 
     conf.for_each_output(
         [&](UserDisplayConfigurationOutput& conf_output)
         {
-            if (!conf_output.connected || conf_output.modes.empty() ||
-                available_outputs_for_card[conf_output.card_id] == 0)
+            if (!conf_output.connected || conf_output.modes.empty())
             {
                 conf_output.used = false;
                 conf_output.power_mode = default_power_state;
@@ -95,8 +85,55 @@ void mg::DefaultDisplayConfigurationPolicy::apply_to(DisplayConfiguration& conf)
             conf_output.current_format = format;
             conf_output.power_mode = default_power_state;
             conf_output.orientation = mir_orientation_normal;
-
-            --available_outputs_for_card[conf_output.card_id];
         });
 }
 
+void mg::SideBySideDisplayConfigurationPolicy::apply_to(graphics::DisplayConfiguration& conf)
+{
+    int max_x = 0;
+
+    conf.for_each_output(
+        [&](mg::UserDisplayConfigurationOutput& conf_output)
+            {
+            if (conf_output.connected && conf_output.modes.size() > 0)
+            {
+                conf_output.used = true;
+                conf_output.top_left = geom::Point{max_x, 0};
+                size_t preferred_mode_index{select_mode_index(conf_output.preferred_mode_index, conf_output.modes)};
+                conf_output.current_mode_index = preferred_mode_index;
+                conf_output.power_mode = mir_power_mode_on;
+                conf_output.orientation = mir_orientation_normal;
+                max_x += conf_output.extents().size.width.as_int();
+            }
+            else
+            {
+                conf_output.used = false;
+                conf_output.power_mode = mir_power_mode_off;
+            }
+            });
+}
+
+
+void mg::SingleDisplayConfigurationPolicy::apply_to(graphics::DisplayConfiguration& conf)
+{
+    bool done{false};
+
+    conf.for_each_output(
+        [&](mg::UserDisplayConfigurationOutput& conf_output)
+            {
+            if (!done && conf_output.connected && conf_output.modes.size() > 0)
+            {
+                conf_output.used = true;
+                conf_output.top_left = geom::Point{0, 0};
+                size_t preferred_mode_index{select_mode_index(conf_output.preferred_mode_index, conf_output.modes)};
+                conf_output.current_mode_index = preferred_mode_index;
+                conf_output.power_mode = mir_power_mode_on;
+                done = true;
+            }
+            else
+            {
+                conf_output.used = false;
+                conf_output.power_mode = mir_power_mode_off;
+            }
+            });
+}
