@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,90 +12,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "input_report.h"
 
-#include "mir/report/legacy_input_report.h"
 #include "mir/logging/logger.h"
 #include "mir/logging/input_timestamp.h"
 
-#include "std/MirLog.h"
-#include <std/Log.h>
-
 #include <linux/input.h>
 
 #include <sstream>
-#include <cstring>
-#include <mutex>
 
 namespace mrl = mir::report::logging;
 namespace ml = mir::logging;
-namespace mrli = mir::report::legacy_input;
-
-namespace
-{
-char const* const component = "android-input";
-
-class LegacyInputReport;
-
-std::mutex mutex;
-std::shared_ptr<LegacyInputReport> the_legacy_input_report;
-
-class LegacyInputReport
-{
-public:
-    LegacyInputReport(std::shared_ptr<ml::Logger> const& logger) :
-        logger(logger)
-    {
-    }
-
-    void log(int prio, char const* buffer)
-    {
-        switch (prio)
-        {
-        case ANDROID_LOG_UNKNOWN:
-        case ANDROID_LOG_DEFAULT:
-        case ANDROID_LOG_VERBOSE:
-        case ANDROID_LOG_DEBUG:
-            logger->log(ml::Severity::debug, buffer, component);
-            break;
-
-        case ANDROID_LOG_INFO:
-            logger->log(ml::Severity::informational, buffer, component);
-            break;
-
-        case ANDROID_LOG_WARN:
-            logger->log(ml::Severity::warning, buffer, component);
-            break;
-
-        case ANDROID_LOG_ERROR:
-            logger->log(ml::Severity::error, buffer, component);
-        };
-    }
-
-private:
-    std::shared_ptr<ml::Logger> const logger;
-};
-
-void my_write_to_log(int prio, char const* buffer)
-{
-    std::unique_lock<std::mutex> lock(mutex);
-    the_legacy_input_report->log(prio, buffer);
-}
-}
-
-
-void mrli::initialize(std::shared_ptr<ml::Logger> const& logger)
-{
-    std::unique_lock<std::mutex> lock(mutex);
-    ::the_legacy_input_report = std::make_shared<LegacyInputReport>(logger);
-
-    mir::write_to_log = my_write_to_log;
-}
-
 
 mrl::InputReport::InputReport(const std::shared_ptr<ml::Logger>& logger)
     : logger(logger)
@@ -177,9 +106,6 @@ std::string print_evdev_code(int type, int code)
         PRINT_EV_ENUM(code, ABS_MT_TRACKING_ID);
         PRINT_EV_ENUM(code, ABS_MT_PRESSURE);
         PRINT_EV_ENUM(code, ABS_MT_DISTANCE);
-        // input.h from android doesn't have these even though modern kernels do.
-        //        PRINT_EV_ENUM(code, ABS_MT_TOOL_X);
-        //        PRINT_EV_ENUM(code, ABS_MT_TOOL_Y);
     }
     return std::to_string(code);
 }
@@ -222,17 +148,6 @@ void mrl::InputReport::published_motion_event(int dest_fd, uint32_t seq_id, int6
     logger->log(ml::Severity::informational, ss.str(), component());
 }
 
-void mrl::InputReport::received_event_finished_signal(int src_fd, uint32_t seq_id)
-{
-    std::stringstream ss;
-
-    ss << "Received event finished"
-       << " seq_id=" << seq_id
-       << " src_fd=" << src_fd;
-
-    logger->log(ml::Severity::informational, ss.str(), component());
-}
-
 void mrl::InputReport::opened_input_device(char const* device_name, char const* input_platform)
 {
     std::stringstream ss;
