@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "display_report.h"
@@ -29,3 +27,4 @@ void mrn::DisplayReport::report_drm_master_failure(int) {}
 void mrn::DisplayReport::report_vt_switch_away_failure() {}
 void mrn::DisplayReport::report_vt_switch_back_failure() {}
 void mrn::DisplayReport::report_egl_configuration(EGLDisplay, EGLConfig) {}
+void mrn::DisplayReport::report_vsync(unsigned int, mir::graphics::Frame const&) {}
