@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored By: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "mir/shell/shell_wrapper.h"
@@ -28,12 +26,13 @@ msh::ShellWrapper::ShellWrapper(std::shared_ptr<Shell> const& wrapped) :
 {
 }
 
-std::shared_ptr<ms::Session> msh::ShellWrapper::open_session(
+auto msh::ShellWrapper::open_session(
     pid_t client_pid,
+    Fd socket_fd,
     std::string const& name,
-    std::shared_ptr<mf::EventSink> const& sink)
+    std::shared_ptr<mf::EventSink> const& sink) -> std::shared_ptr<ms::Session>
 {
-    return wrapped->open_session(client_pid, name, sink);
+    return wrapped->open_session(client_pid, socket_fd, name, sink);
 }
 
 void msh::ShellWrapper::close_session(std::shared_ptr<ms::Session> const& session)
@@ -46,11 +45,21 @@ void msh::ShellWrapper::focus_next_session()
     wrapped->focus_next_session();
 }
 
+void msh::ShellWrapper::focus_prev_session()
+{
+    wrapped->focus_prev_session();
+}
+
 std::shared_ptr<ms::Session> msh::ShellWrapper::focused_session() const
 {
     return wrapped->focused_session();
 }
 
+void msh::ShellWrapper::set_popup_grab_tree(std::shared_ptr<scene::Surface> const& surface)
+{
+    wrapped->set_popup_grab_tree(surface);
+}
+
 void msh::ShellWrapper::set_focus_to(
     std::shared_ptr<scene::Session> const& focus_session,
     std::shared_ptr<scene::Surface> const& focus_surface)
@@ -78,20 +87,32 @@ void msh::ShellWrapper::stop_prompt_session(std::shared_ptr<ms::PromptSession> c
     wrapped->stop_prompt_session(prompt_session);
 }
 
-mf::SurfaceId msh::ShellWrapper::create_surface(
+auto msh::ShellWrapper::create_surface(
     std::shared_ptr<ms::Session> const& session,
-    ms::SurfaceCreationParameters const& params,
-    std::shared_ptr<mf::EventSink> const& sink)
+    wayland::Weak<frontend::WlSurface> const& wayland_surface,
+    SurfaceSpecification const& params,
+    std::shared_ptr<ms::SurfaceObserver> const& observer,
+    Executor* observer_executor) -> std::shared_ptr<ms::Surface>
 {
-    return wrapped->create_surface(session, params, sink);
+    return wrapped->create_surface(session, wayland_surface, params, observer, observer_executor);
 }
 
-void msh::ShellWrapper::modify_surface(std::shared_ptr<scene::Session> const& session, std::shared_ptr<scene::Surface> const& surface, SurfaceSpecification const& modifications)
+void msh::ShellWrapper::surface_ready(std::shared_ptr<ms::Surface> const& surface)
+{
+    wrapped->surface_ready(surface);
+}
+
+void msh::ShellWrapper::modify_surface(
+    std::shared_ptr<scene::Session> const& session,
+    std::shared_ptr<scene::Surface> const& surface,
+    SurfaceSpecification const& modifications)
 {
     wrapped->modify_surface(session, surface, modifications);
 }
 
-void msh::ShellWrapper::destroy_surface(std::shared_ptr<ms::Session> const& session, mf::SurfaceId surface)
+void msh::ShellWrapper::destroy_surface(
+    std::shared_ptr<ms::Session> const& session,
+    std::shared_ptr<ms::Surface> const& surface)
 {
     wrapped->destroy_surface(session, surface);
 }
@@ -99,7 +120,7 @@ void msh::ShellWrapper::destroy_surface(std::shared_ptr<ms::Session> const& sess
 int msh::ShellWrapper::set_surface_attribute(
     std::shared_ptr<ms::Session> const& session,
     std::shared_ptr<ms::Surface> const& surface,
-    MirSurfaceAttrib attrib,
+    MirWindowAttrib attrib,
     int value)
 {
     return wrapped->set_surface_attribute(session, surface, attrib, value);
@@ -107,11 +128,44 @@ int msh::ShellWrapper::set_surface_attribute(
 
 int msh::ShellWrapper::get_surface_attribute(
     std::shared_ptr<ms::Surface> const& surface,
-    MirSurfaceAttrib attrib)
+    MirWindowAttrib attrib)
 {
     return wrapped->get_surface_attribute(surface, attrib);
 }
 
+void msh::ShellWrapper::raise_surface(
+    std::shared_ptr<ms::Session> const& session,
+    std::shared_ptr<ms::Surface> const& surface,
+    uint64_t timestamp)
+{
+    wrapped->raise_surface(session, surface, timestamp);
+}
+
+void msh::ShellWrapper::request_drag_and_drop(
+    std::shared_ptr<ms::Session> const& session,
+    std::shared_ptr<ms::Surface> const& surface,
+    uint64_t timestamp)
+{
+    wrapped->request_drag_and_drop(session, surface, timestamp);
+}
+
+void msh::ShellWrapper::request_move(
+    std::shared_ptr<ms::Session> const& session,
+    std::shared_ptr<ms::Surface> const& surface,
+    uint64_t timestamp)
+{
+    wrapped->request_move(session, surface, timestamp);
+}
+
+void msh::ShellWrapper::request_resize(
+    std::shared_ptr<scene::Session> const& session,
+    std::shared_ptr<scene::Surface> const& surface,
+    uint64_t timestamp,
+    MirResizeEdge edge)
+{
+    wrapped->request_resize(session, surface, timestamp, edge);
+}
+
 void msh::ShellWrapper::add_display(geometry::Rectangle const& area)
 {
     wrapped->add_display(area);
@@ -141,3 +195,13 @@ void msh::ShellWrapper::raise(SurfaceSet const& surfaces)
 {
     return wrapped->raise(surfaces);
 }
+
+void msh::ShellWrapper::set_drag_and_drop_handle(std::vector<uint8_t> const& handle)
+{
+    wrapped->set_drag_and_drop_handle(handle);
+}
+
+void msh::ShellWrapper::clear_drag_and_drop_handle()
+{
+    wrapped->clear_drag_and_drop_handle();
+}
