@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
  */
 
 #ifndef MIR_TEST_DOUBLES_NULL_EVENT_SINK_H_
@@ -36,11 +34,16 @@ namespace doubles
 {
 struct NullEventSink : public frontend::EventSink
 {
-    void handle_event(MirEvent const&) {}
-    void handle_lifecycle_event(MirLifecycleState) {}
-    void handle_display_config_change(graphics::DisplayConfiguration const&) {}
-    void send_ping(int32_t) {}
-    void send_buffer(frontend::BufferStreamId, graphics::Buffer&, graphics::BufferIpcMsgType) {}
+    void handle_event(EventUPtr&&) override {}
+    void handle_lifecycle_event(MirLifecycleState) override {}
+    void handle_display_config_change(graphics::DisplayConfiguration const&) override {}
+    void handle_error(ClientVisibleError const&) override {}
+    void send_ping(int32_t) override {}
+    void send_buffer(frontend::BufferStreamId, graphics::Buffer&) override {}
+    void handle_input_config_change(MirInputConfig const&) override {}
+    void add_buffer(graphics::Buffer&) override {}
+    void update_buffer(graphics::Buffer&) override {}
+    void error_buffer(geometry::Size, MirPixelFormat, std::string const&) override {}
 };
 
 std::unique_ptr<frontend::EventSink> null_sink_factory(std::shared_ptr<frontend::MessageSender> const&);
