@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,18 +12,18 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by:
- * Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
-#include "mir_test_doubles/mock_drm.h"
+#include "mir/test/doubles/mock_drm.h"
+#include "mir_test_framework/open_wrapper.h"
 #include "mir/geometry/size.h"
 #include <gtest/gtest.h>
 
 #include <stdexcept>
 #include <unistd.h>
 #include <dlfcn.h>
+#include <system_error>
+#include <boost/throw_exception.hpp>
 
 namespace mtd=mir::test::doubles;
 namespace geom = mir::geometry;
@@ -155,7 +155,8 @@ void mtd::FakeDRMResources::add_connector(uint32_t connector_id,
                                           uint32_t encoder_id,
                                           std::vector<drmModeModeInfo>& modes,
                                           std::vector<uint32_t>& possible_encoder_ids,
-                                          geom::Size const& physical_size)
+                                          geom::Size const& physical_size,
+                                          drmModeSubPixel subpixel_arrangement)
 {
     drmModeConnector connector = drmModeConnector();
 
@@ -169,6 +170,7 @@ void mtd::FakeDRMResources::add_connector(uint32_t connector_id,
     connector.count_encoders = possible_encoder_ids.size();
     connector.mmWidth = physical_size.width.as_uint32_t();
     connector.mmHeight = physical_size.height.as_uint32_t();
+    connector.subpixel = subpixel_arrangement;
 
     connectors.push_back(connector);
 }
@@ -216,6 +218,11 @@ drmModeModeInfo mtd::FakeDRMResources::create_mode(uint16_t hdisplay, uint16_t v
     mode.clock = clock;
     mode.htotal = htotal;
     mode.vtotal = vtotal;
+
+    uint32_t total = htotal;
+    total *= vtotal;  // extend to 32 bits
+    mode.vrefresh = clock * 1000UL / total;
+
     if (preferred)
         mode.type |= DRM_MODE_TYPE_PREFERRED;
 
@@ -223,35 +230,134 @@ drmModeModeInfo mtd::FakeDRMResources::create_mode(uint16_t hdisplay, uint16_t v
 }
 
 mtd::MockDRM::MockDRM()
+    : open_interposer{mir_test_framework::add_open_handler(
+        [this](char const* path, int flags, std::optional<mode_t>) -> std::optional<int>
+        {
+            char const* const drm_prefix = "/dev/dri/";
+            if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
+            {
+                // I don't think we need to be able to distinguish based on mode. ppc64el (at least) *does*
+                // call the 3-parameter open()
+                return this->open(path, flags);
+            }
+            return std::nullopt;
+        })}
 {
     using namespace testing;
     assert(global_mock == NULL && "Only one mock object per process is allowed");
 
     global_mock = this;
 
-    ON_CALL(*this, open(_,_,_))
-    .WillByDefault(Return(fake_drm.fd()));
+    memset(&empty_object_props, 0, sizeof(empty_object_props));
+
+    ON_CALL(*this, open(_,_))
+        .WillByDefault(
+            WithArg<0>(
+                Invoke(
+                    [this](char const* device_path)
+                    {
+                        auto fd = fake_drms[device_path].fd();
+                        fd_to_drm.insert({fd, fake_drms[device_path]});
+                        return fd;
+                    })));
 
     ON_CALL(*this, drmOpen(_,_))
-    .WillByDefault(Return(fake_drm.fd()));
+        .WillByDefault(
+            InvokeWithoutArgs(
+                [this]()
+                {
+                    auto fd = fake_drms["/dev/dri/card0"].fd();
+                    fd_to_drm.insert({fd, fake_drms["/dev/dri/card0"]});
+                    return fd;
+                }));
 
     ON_CALL(*this, drmModeGetResources(_))
-    .WillByDefault(Return(fake_drm.resources_ptr()));
+        .WillByDefault(
+            Invoke(
+                [this](int fd)
+                {
+                    return fd_to_drm.at(fd).resources_ptr();
+                }));
 
     ON_CALL(*this, drmModeGetCrtc(_, _))
-    .WillByDefault(WithArgs<1>(Invoke(&fake_drm, &FakeDRMResources::find_crtc)));
+        .WillByDefault(
+            Invoke(
+                [this](int fd, uint32_t crtc_id)
+                {
+                    return fd_to_drm.at(fd).find_crtc(crtc_id);
+                }));
 
     ON_CALL(*this, drmModeGetEncoder(_, _))
-    .WillByDefault(WithArgs<1>(Invoke(&fake_drm, &FakeDRMResources::find_encoder)));
+        .WillByDefault(
+            Invoke(
+                [this](int fd, uint32_t encoder_id)
+                {
+                    return fd_to_drm.at(fd).find_encoder(encoder_id);
+                }));
 
     ON_CALL(*this, drmModeGetConnector(_, _))
-    .WillByDefault(WithArgs<1>(Invoke(&fake_drm, &FakeDRMResources::find_connector)));
+        .WillByDefault(
+            Invoke(
+                [this](int fd, uint32_t connector_id)
+                {
+                    return fd_to_drm.at(fd).find_connector(connector_id);
+                }));
+
+    ON_CALL(*this, drmModeObjectGetProperties(_, _, _))
+        .WillByDefault(Return(&empty_object_props));
 
     ON_CALL(*this, drmSetInterfaceVersion(_, _))
-    .WillByDefault(Return(0));
+        .WillByDefault(
+            Invoke(
+                [this](auto fd, auto version)
+                {
+                    if (version->drm_di_major != 1 || version->drm_di_minor != 4)
+                    {
+                        ADD_FAILURE() << "We should only ever request DRM version 1.4";
+                        errno = EINVAL;
+                        return -1;
+                    }
+
+                    fd_to_drm.at(fd).drm_setversion_called = true;
+                    return 0;
+                }));
 
     ON_CALL(*this, drmGetBusid(_))
-    .WillByDefault(WithoutArgs(Invoke([]{ return static_cast<char*>(malloc(10)); })));
+        .WillByDefault(
+            Invoke(
+                [this](auto fd) -> char*
+                {
+                    if (!fd_to_drm.at(fd).drm_setversion_called)
+                    {
+                        return nullptr;
+                    }
+                    return static_cast<char*>(malloc(10));
+                }));
+
+    ON_CALL(*this, drmFreeBusid(_))
+        .WillByDefault(WithArg<0>(Invoke([&](const char* busid) { free(const_cast<char*>(busid)); })));
+
+    ON_CALL(*this, drmGetPrimaryDeviceNameFromFd(_))
+        .WillByDefault(InvokeWithoutArgs([]() { return strdup("/dev/dri/card0"); }));
+
+    static drmVersion const version{
+        1,
+        2,
+        3,
+        static_cast<int>(strlen("mock_driver")),
+        const_cast<char*>("mock_driver"),
+        static_cast<int>(strlen("1 Jan 1970")),
+        const_cast<char*>("1 Jan 1970"),
+        static_cast<int>(strlen("Not really a driver")),
+        const_cast<char*>("Not really a driver")
+    };
+    ON_CALL(*this, drmGetVersion(_))
+        .WillByDefault(Return(const_cast<drmVersionPtr>(&version)));
+
+    ON_CALL(*this, drmCheckModesettingSupported(NotNull()))
+        .WillByDefault(Return(0));
+    ON_CALL(*this, drmCheckModesettingSupported(IsNull()))
+        .WillByDefault(Return(-EINVAL));
 }
 
 mtd::MockDRM::~MockDRM() noexcept
@@ -259,16 +365,135 @@ mtd::MockDRM::~MockDRM() noexcept
     global_mock = nullptr;
 }
 
+void mtd::MockDRM::add_crtc(char const *device, uint32_t id, drmModeModeInfo mode)
+{
+    fake_drms[device].add_crtc(id, mode);
+}
+
+void mtd::MockDRM::add_encoder(
+    char const *device,
+    uint32_t encoder_id,
+    uint32_t crtc_id,
+    uint32_t possible_crtcs_mask)
+{
+    fake_drms[device].add_encoder(encoder_id, crtc_id, possible_crtcs_mask);
+}
+
+void mtd::MockDRM::prepare(char const *device)
+{
+    fake_drms[device].prepare();
+}
+
+void mtd::MockDRM::reset(char const *device)
+{
+    fake_drms[device].reset();
+}
+
+void mtd::MockDRM::generate_event_on(char const *device)
+{
+    auto const fd = fake_drms[device].write_fd();
+
+    if (write(fd, "a", 1) != 1)
+    {
+        BOOST_THROW_EXCEPTION(
+            std::system_error(errno, std::system_category(), "Failed to make fake DRM event"));
+    }
+}
+
+void mtd::MockDRM::consume_event_on(char const* device)
+{
+    auto const fd = fake_drms[device].fd();
+
+    char consume_buf;
+    if (::read(fd, &consume_buf, 1) != 1)
+    {
+        BOOST_THROW_EXCEPTION(
+            std::system_error(errno, std::system_category(), "Failed to consume fake DRM event"));
+    }
+}
+
+void mtd::MockDRM::add_connector(
+    char const *device,
+    uint32_t connector_id,
+    uint32_t type,
+    drmModeConnection connection,
+    uint32_t encoder_id,
+    std::vector<drmModeModeInfo> &modes,
+    std::vector<uint32_t> &possible_encoder_ids,
+    geometry::Size const &physical_size,
+    drmModeSubPixel subpixel_arrangement)
+{
+    fake_drms[device].add_connector(
+        connector_id,
+        type,
+        connection,
+        encoder_id,
+        modes,
+        possible_encoder_ids,
+        physical_size,
+        subpixel_arrangement);
+}
+
+MATCHER_P2(IsFdOfDevice, devname, fds, "")
+{
+    return std::find(
+        fds.begin(),
+        fds.end(),
+        arg) != fds.end();
+}
+
+class mtd::MockDRM::IsFdOfDeviceMatcher : public ::testing::MatcherInterface<int>
+{
+public:
+    IsFdOfDeviceMatcher(char const* device)
+        : device{device}
+    {
+    }
+
+    void DescribeTo(::std::ostream *os) const override
+    {
+
+        *os
+            << "Is an fd of DRM device "
+            << device
+            << " (one of: "
+            << ::testing::PrintToString(fds_for_device(device.c_str()))
+            << ")";
+    }
+
+    bool MatchAndExplain(int x, testing::MatchResultListener *listener) const override
+    {
+        testing::Matcher<std::vector<int>> matcher = testing::Contains(x);
+        return matcher.MatchAndExplain(fds_for_device(device.c_str()), listener);
+    }
+
+private:
+    std::vector<int> fds_for_device(char const* device) const
+    {
+        std::vector<int> device_fds;
+        for (auto const& pair : global_mock->fd_to_drm)
+        {
+            if (&global_mock->fake_drms[device] == &pair.second)
+            {
+                device_fds.push_back(pair.first);
+            }
+        }
+        return device_fds;
+    }
+
+    std::string const device;
+};
+
+testing::Matcher<int> mtd::IsFdOfDevice(char const* device)
+{
+    return ::testing::MakeMatcher(new mtd::MockDRM::IsFdOfDeviceMatcher(device));
+}
+
 int drmOpen(const char *name, const char *busid)
 {
     return global_mock->drmOpen(name, busid);
 }
 
-int drmClose(int fd)
-{
-    return global_mock->drmClose(fd);
-}
-
 int drmIoctl(int fd, unsigned long request, void *arg)
 {
     return global_mock->drmIoctl(fd, request, arg);
@@ -289,6 +514,21 @@ drmModeEncoderPtr drmModeGetEncoder(int fd, uint32_t encoder_id)
     return global_mock->drmModeGetEncoder(fd, encoder_id);
 }
 
+drmModePlaneResPtr drmModeGetPlaneResources(int fd)
+{
+    return global_mock->drmModeGetPlaneResources(fd);
+}
+
+drmModePlanePtr drmModeGetPlane(int fd, uint32_t plane_id)
+{
+    return global_mock->drmModeGetPlane(fd, plane_id);
+}
+
+drmModeObjectPropertiesPtr drmModeObjectGetProperties(int fd, uint32_t id, uint32_t type)
+{
+    return global_mock->drmModeObjectGetProperties(fd, id, type);
+}
+
 drmModeCrtcPtr drmModeGetCrtc(int fd, uint32_t crtcId)
 {
     return global_mock->drmModeGetCrtc(fd, crtcId);
@@ -302,11 +542,58 @@ int drmModeSetCrtc(int fd, uint32_t crtcId, uint32_t bufferId,
                                        connectors, count, mode);
 }
 
+int drmModeCrtcGetGamma(int fd, uint32_t crtc_id, uint32_t size,
+                        uint16_t* red, uint16_t* green, uint16_t* blue)
+{
+    return global_mock->drmModeCrtcGetGamma(fd, crtc_id, size, red, green, blue);
+}
+
+int drmModeCrtcSetGamma(int fd, uint32_t crtc_id, uint32_t size,
+                        uint16_t* red, uint16_t* green, uint16_t* blue)
+{
+    return global_mock->drmModeCrtcSetGamma(fd, crtc_id, size, red, green, blue);
+}
+
 void drmModeFreeResources(drmModeResPtr ptr)
 {
     global_mock->drmModeFreeResources(ptr);
 }
 
+void drmModeFreeProperty(drmModePropertyPtr ptr)
+{
+    global_mock->drmModeFreeProperty(ptr);
+}
+
+int drmGetCap(int fd, uint64_t capability, uint64_t *value)
+{
+    return global_mock->drmGetCap(fd, capability, value);
+}
+
+drmVersionPtr drmGetVersion(int fd)
+{
+    return global_mock->drmGetVersion(fd);
+}
+
+void drmFreeVersion(drmVersionPtr version)
+{
+    return global_mock->drmFreeVersion(version);
+}
+
+int drmSetClientCap(int fd, uint64_t capability, uint64_t value)
+{
+    return global_mock->drmSetClientCap(fd, capability, value);
+}
+
+drmModePropertyPtr drmModeGetProperty(int fd, uint32_t propertyId)
+{
+    return global_mock->drmModeGetProperty(fd, propertyId);
+}
+
+int drmModeConnectorSetProperty(int fd, uint32_t connector_id, uint32_t property_id, uint64_t value)
+{
+    return global_mock->drmModeConnectorSetProperty(fd, connector_id, property_id, value);
+}
+
 void drmModeFreeConnector(drmModeConnectorPtr ptr)
 {
     global_mock->drmModeFreeConnector(ptr);
@@ -322,6 +609,21 @@ void drmModeFreeCrtc(drmModeCrtcPtr ptr)
     global_mock->drmModeFreeCrtc(ptr);
 }
 
+void drmModeFreePlaneResources(drmModePlaneResPtr ptr)
+{
+    global_mock->drmModeFreePlaneResources(ptr);
+}
+
+void drmModeFreePlane(drmModePlanePtr ptr)
+{
+    global_mock->drmModeFreePlane(ptr);
+}
+
+void drmModeFreeObjectProperties(drmModeObjectPropertiesPtr ptr)
+{
+    global_mock->drmModeFreeObjectProperties(ptr);
+}
+
 int drmModeAddFB(int fd, uint32_t width, uint32_t height,
                  uint8_t depth, uint8_t bpp, uint32_t pitch,
                  uint32_t bo_handle, uint32_t *buf_id)
@@ -331,16 +633,39 @@ int drmModeAddFB(int fd, uint32_t width, uint32_t height,
                                      pitch, bo_handle, buf_id);
 }
 
+#ifdef MIR_DRMMODEADDFB_HAS_CONST_SIGNATURE
+int drmModeAddFB2(int fd, uint32_t width, uint32_t height,
+                  uint32_t pixel_format, uint32_t const bo_handles[4],
+                  uint32_t const pitches[4], uint32_t const offsets[4],
+                  uint32_t* buf_id, uint32_t flags)
+#else
 int drmModeAddFB2(int fd, uint32_t width, uint32_t height,
                   uint32_t pixel_format, uint32_t bo_handles[4],
                   uint32_t pitches[4], uint32_t offsets[4],
                   uint32_t *buf_id, uint32_t flags)
+#endif
 {
     return global_mock->drmModeAddFB2(fd, width, height, pixel_format,
                                       bo_handles, pitches, offsets,
                                       buf_id, flags);
 }
 
+int drmModeAddFB2WithModifiers(
+    int fd,
+    uint32_t width,
+    uint32_t height,
+    uint32_t fourcc,
+    uint32_t const handles[4],
+    uint32_t const pitches[4],
+    uint32_t const offsets[4],
+    uint64_t const modifiers[4],
+    uint32_t *buf_id,
+    uint32_t flags)
+{
+    return global_mock->drmModeAddFB2WithModifiers(
+        fd, width, height, fourcc, handles, pitches, offsets, modifiers, buf_id, flags);
+}
+
 int drmModeRmFB(int fd, uint32_t bufferId)
 {
     return global_mock->drmModeRmFB(fd, bufferId);
@@ -379,6 +704,11 @@ int drmPrimeFDToHandle(int fd, int prime_fd, uint32_t *handle)
     return global_mock->drmPrimeFDToHandle(fd, prime_fd, handle);
 }
 
+int drmIsMaster(int fd)
+{
+    return global_mock->drmIsMaster(fd);
+}
+
 int drmSetMaster(int fd)
 {
     return global_mock->drmSetMaster(fd);
@@ -409,57 +739,22 @@ char* drmGetBusid(int fd)
     return global_mock->drmGetBusid(fd);
 }
 
-// We need to wrap open as we sometimes open() the DRM device
-// We need to explicitly mark this as C because we don't match the
-// libc header; we only care about the three-parameter version
-extern "C"
+void drmFreeBusid(const char* busid)
 {
-int open(char const* path, int flags, mode_t mode)
-{
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open) = dlsym(RTLD_NEXT, "open");
-
-    return (*real_open)(path, flags, mode);
+    return global_mock->drmFreeBusid(busid);
 }
 
-int open64(char const* path, int flags, mode_t mode)
+char* drmGetDeviceNameFromFd(int fd)
 {
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open64)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open64) = dlsym(RTLD_NEXT, "open64");
-
-    return (*real_open64)(path, flags, mode);
+    return global_mock->drmGetDeviceNameFromFd(fd);
 }
 
-int __open(char const* path, int flags, mode_t mode)
+char* drmGetPrimaryDeviceNameFromFd(int fd)
 {
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open) = dlsym(RTLD_NEXT, "__open");
-
-    return (*real_open)(path, flags, mode);
+    return global_mock->drmGetPrimaryDeviceNameFromFd(fd);
 }
 
-int __open64(char const* path, int flags, mode_t mode)
+int drmCheckModesettingSupported(char const* busid)
 {
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open64)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open64) = dlsym(RTLD_NEXT, "__open64");
-
-    return (*real_open64)(path, flags, mode);
-}
-
+    return global_mock->drmCheckModesettingSupported(busid);
 }
