@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
 #include "mir/test/doubles/triggered_main_loop.h"
@@ -67,8 +65,10 @@ void mtd::TriggeredMainLoop::trigger_pending_fds()
     {
         for (auto const& item : fd_callbacks)
         {
-            FD_ISSET(item.fd, &read_fds);
-            item.callback(item.fd);
+            if (FD_ISSET(item.fd, &read_fds))
+            {
+                item.callback(item.fd);
+            }
         }
     }
 }
@@ -79,10 +79,17 @@ void mtd::TriggeredMainLoop::enqueue(void const* owner, ServerAction const& acti
     actions.push_back(action);
 }
 
+void mtd::TriggeredMainLoop::enqueue_with_guaranteed_execution(ServerAction const& action)
+{
+    base::enqueue(nullptr, action);
+    actions.push_back(action);
+}
+
 void mtd::TriggeredMainLoop::trigger_server_actions()
 {
     for (auto const& action : actions)
         action();
+    actions.clear();
 }
 
 std::unique_ptr<mir::time::Alarm>
@@ -98,3 +105,18 @@ void mtd::TriggeredMainLoop::fire_all_alarms()
     for(auto const& callback : timeout_callbacks)
         callback();
 }
+
+void mtd::TriggeredMainLoop::spawn(std::function<void()>&& work)
+{
+    base::spawn(std::function<void()>{work});
+    this->work.emplace_back(std::move(work));
+}
+
+void mtd::TriggeredMainLoop::trigger_spawned_work()
+{
+    for (auto const& action : work)
+    {
+        action();
+    }
+    work.clear();
+}
