@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,12 +12,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Thomas Voss <thomas.voss@canonical.com>
- *              Thomas Guest <thomas.guest@canonical.com>
  */
 
 #include "mir_test_framework/process.h"
+#include "mir/log.h"
 
 #include <errno.h>
 #include <signal.h>
@@ -25,6 +23,7 @@
 
 #include <boost/exception/errinfo_errno.hpp>
 
+#include <system_error>
 #include <cassert>
 #include <chrono>
 #include <ostream>
@@ -45,10 +44,11 @@ void signal_process(pid_t pid, int signum)
     if (::kill(pid, signum) != 0)
     {
         BOOST_THROW_EXCEPTION(
-            ::boost::enable_error_info(std::runtime_error("Failed to kill process."))
+            ::boost::enable_error_info(std::system_error(errno,
+                                                         std::system_category(),
+                                                         "Failed to kill process."))
             << errinfo_pid(pid)
-            << errinfo_signum(signum)
-            << boost::errinfo_errno(errno));
+            << errinfo_signum(signum));
     }
 }
 }
@@ -71,6 +71,11 @@ bool mtf::Result::signalled() const
     return reason == TerminationReason::child_terminated_by_signal;
 }
 
+bool mtf::Result::exited_normally() const
+{
+    return reason == TerminationReason::child_terminated_normally;
+}
+
 mtf::Process::Process(pid_t pid)
     : pid(pid)
     , terminated(false)
@@ -88,9 +93,13 @@ mtf::Process::~Process()
             terminate();
             wait_for_termination();
         }
-        catch (std::exception const &)
+        catch (std::exception const&)
         {
-            // Ignore a failure to signal the process.
+            mir::log(
+                mir::logging::Severity::error,
+               "mir_test_framework",
+               std::current_exception(),
+               "Failed to signal process");
         }
     }
 }
@@ -122,12 +131,17 @@ mtf::Result mtf::Process::wait_for_termination(const std::chrono::milliseconds&
                     result.signal = WTERMSIG(status);
                     break;
                 }
+                else {
+                    terminated = true;
+                    result.reason = TerminationReason::unknown;
+                    break;
+                }
             }
             else if (rc == 0)
             {
                 if (std::chrono::steady_clock::now() < tp)
                 {
-                    std::this_thread::yield();
+                    std::this_thread::sleep_for(std::chrono::milliseconds(10));
                     continue;
                 }
                 else
@@ -200,7 +214,26 @@ std::ostream& print_reason(std::ostream & out, mtf::TerminationReason reason)
 
 std::ostream& print_signal(std::ostream& out, int signal)
 {
-    out << "signal(" << signal << ")";
+    switch(signal)
+    {
+    case SIGQUIT:
+        out << "SIGQUIT";
+        break;
+    case SIGABRT:
+        out << "SIGABRT";
+        break;
+    case SIGFPE:
+        out << "SIGFPE";
+        break;
+    case SIGSEGV:
+        out << "SIGSEGV";
+        break;
+    case SIGBUS:
+        out << "SIGBUS";
+        break;
+    default:
+        out << "signal(" + std::to_string(signal) + ")";
+    }
     return out;
 }
 
@@ -222,15 +255,15 @@ std::ostream& mtf::operator<<(std::ostream& out, const mtf::Result& result)
 {
     out << "process::Result(";
     print_reason(out, result.reason);
-    out << ", ";
     if (result.signalled())
     {
-        print_signal(out, result.signal);
         out << ", ";
+        print_signal(out, result.signal);
     }
 
     if (result.reason == TerminationReason::child_terminated_normally)
     {
+        out << ", ";
         print_exit_code(out, result.exit_code);
     }
 
