@@ -1,8 +1,8 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,28 +12,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
 #include "src/server/compositor/default_display_buffer_compositor.h"
-#include "mir/compositor/display_buffer_compositor.h"
 #include "src/server/report/null_report_factory.h"
 #include "mir/compositor/scene.h"
-#include "mir/compositor/renderer.h"
+#include "mir/renderer/renderer.h"
 #include "mir/geometry/rectangle.h"
-#include "mir_test_doubles/mock_renderer.h"
-#include "mir_test/fake_shared.h"
-#include "mir_test_doubles/mock_display_buffer.h"
-#include "mir_test_doubles/mock_renderable.h"
-#include "mir_test_doubles/fake_renderable.h"
-#include "mir_test_doubles/stub_display_buffer.h"
-#include "mir_test_doubles/mock_buffer.h"
-#include "mir_test_doubles/stub_buffer.h"
-#include "mir_test_doubles/mock_compositor_report.h"
-#include "mir_test_doubles/mock_scene.h"
-#include "mir_test_doubles/stub_scene.h"
-#include "mir_test_doubles/stub_scene_element.h"
+#include "mir/test/doubles/mock_renderer.h"
+#include "mir/test/fake_shared.h"
+#include "mir/test/doubles/mock_display_buffer.h"
+#include "mir/test/doubles/fake_renderable.h"
+#include "mir/test/doubles/mock_compositor_report.h"
+#include "mir/test/doubles/stub_scene_element.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -50,6 +41,8 @@ namespace mtd = mir::test::doubles;
 namespace
 {
 
+glm::mat2 const no_transformation(1);
+
 struct StubSceneElement : mc::SceneElement
 {
     StubSceneElement(std::shared_ptr<mg::Renderable> const& renderable) :
@@ -57,21 +50,16 @@ struct StubSceneElement : mc::SceneElement
     {
     }
 
-    std::shared_ptr<mir::graphics::Renderable> renderable() const
+    std::shared_ptr<mir::graphics::Renderable> renderable() const override
     {
         return renderable_;
     }
 
-    bool is_a_surface() const
-    {
-        return true;
-    }
-
-    void rendered()
+    void rendered() override
     {
     }
 
-    void occluded()
+    void occluded() override
     {
     }
 
@@ -96,11 +84,11 @@ struct DefaultDisplayBufferCompositor : public testing::Test
        fullscreen(std::make_shared<mtd::FakeRenderable>(screen))
     {
         using namespace testing;
-        ON_CALL(display_buffer, orientation())
-            .WillByDefault(Return(mir_orientation_normal));
+        ON_CALL(display_buffer, transformation())
+            .WillByDefault(Return(no_transformation));
         ON_CALL(display_buffer, view_area())
             .WillByDefault(Return(screen));
-        ON_CALL(display_buffer, post_renderables_if_optimizable(_))
+        ON_CALL(display_buffer, overlay(_))
             .WillByDefault(Return(false));
     }
 
@@ -120,12 +108,6 @@ TEST_F(DefaultDisplayBufferCompositor, render)
         .Times(1);
     EXPECT_CALL(display_buffer, view_area())
         .Times(AtLeast(1));
-    EXPECT_CALL(display_buffer, make_current())
-        .Times(1);
-    EXPECT_CALL(display_buffer, gl_swap_buffers())
-        .Times(1);
-    EXPECT_CALL(display_buffer, flip())
-        .Times(1);
 
     mc::DefaultDisplayBufferCompositor compositor(
         display_buffer,
@@ -142,9 +124,11 @@ TEST_F(DefaultDisplayBufferCompositor, optimization_skips_composition)
     Sequence seq;
     EXPECT_CALL(*report, began_frame(_))
         .InSequence(seq);
-    EXPECT_CALL(display_buffer, post_renderables_if_optimizable(_))
+    EXPECT_CALL(display_buffer, overlay(_))
         .InSequence(seq)
         .WillOnce(Return(true));
+    EXPECT_CALL(*report, renderables_in_frame(_,_))
+        .InSequence(seq);
     EXPECT_CALL(mock_renderer, suspend())
         .InSequence(seq);
     EXPECT_CALL(*report, rendered_frame(_))
@@ -170,9 +154,11 @@ TEST_F(DefaultDisplayBufferCompositor, rendering_reports_everything)
     Sequence seq;
     EXPECT_CALL(*report, began_frame(_))
         .InSequence(seq);
-    EXPECT_CALL(display_buffer, post_renderables_if_optimizable(_))
+    EXPECT_CALL(display_buffer, overlay(_))
         .InSequence(seq)
         .WillOnce(Return(false));
+    EXPECT_CALL(*report, renderables_in_frame(_,_))
+        .InSequence(seq);
     EXPECT_CALL(*report, rendered_frame(_))
         .InSequence(seq);
     EXPECT_CALL(*report, finished_frame(_))
@@ -194,18 +180,50 @@ TEST_F(DefaultDisplayBufferCompositor, calls_renderer_in_sequence)
     Sequence render_seq;
     EXPECT_CALL(mock_renderer, suspend())
         .Times(0);
-    EXPECT_CALL(display_buffer, make_current())
+    EXPECT_CALL(display_buffer, transformation())
+        .WillOnce(Return(no_transformation));
+    EXPECT_CALL(mock_renderer, set_output_transform(no_transformation))
         .InSequence(render_seq);
-    EXPECT_CALL(display_buffer, orientation())
-        .InSequence(render_seq)
-        .WillOnce(Return(mir_orientation_normal));
-    EXPECT_CALL(mock_renderer, set_rotation(_))
+    EXPECT_CALL(mock_renderer, set_viewport(screen))
         .InSequence(render_seq);
     EXPECT_CALL(mock_renderer, render(ContainerEq(mg::RenderableList{big, small})))
         .InSequence(render_seq);
-    EXPECT_CALL(display_buffer, gl_swap_buffers())
+
+    mc::DefaultDisplayBufferCompositor compositor(
+        display_buffer,
+        mt::fake_shared(mock_renderer),
+        mr::null_compositor_report());
+
+    compositor.composite(make_scene_elements({
+        big,
+        small
+    }));
+}
+
+TEST_F(DefaultDisplayBufferCompositor, rotates_viewport)
+{   // Regression test for LP: #1643488
+    using namespace testing;
+
+    glm::mat2 const rotate_left( 0, 1,  // transposed
+                                -1, 0);
+
+    geom::Rectangle const rotated_screen{
+        screen.top_left,
+        {screen.size.height.as_int(), screen.size.width.as_int()}};
+
+    ON_CALL(display_buffer, view_area())
+        .WillByDefault(Return(rotated_screen));
+
+    Sequence render_seq;
+    EXPECT_CALL(mock_renderer, suspend())
+        .Times(0);
+    EXPECT_CALL(display_buffer, transformation())
+        .WillOnce(Return(rotate_left));
+    EXPECT_CALL(mock_renderer, set_output_transform(rotate_left))
         .InSequence(render_seq);
-    EXPECT_CALL(display_buffer, flip())
+    EXPECT_CALL(mock_renderer, set_viewport(rotated_screen))
+        .InSequence(render_seq);
+    EXPECT_CALL(mock_renderer, render(ContainerEq(mg::RenderableList{big, small})))
         .InSequence(render_seq);
 
     mc::DefaultDisplayBufferCompositor compositor(
@@ -224,47 +242,39 @@ TEST_F(DefaultDisplayBufferCompositor, optimization_toggles_seamlessly)
     using namespace testing;
     ON_CALL(display_buffer, view_area())
         .WillByDefault(Return(screen));
-    ON_CALL(display_buffer, orientation())
-        .WillByDefault(Return(mir_orientation_normal));
+    ON_CALL(display_buffer, transformation())
+        .WillByDefault(Return(no_transformation));
 
     Sequence seq;
-    EXPECT_CALL(display_buffer, post_renderables_if_optimizable(_))
+    EXPECT_CALL(display_buffer, overlay(_))
         .InSequence(seq)
         .WillOnce(Return(false));
 
-    EXPECT_CALL(display_buffer, make_current())
+    EXPECT_CALL(display_buffer, transformation())
         .InSequence(seq);
-    EXPECT_CALL(display_buffer, orientation())
+    EXPECT_CALL(mock_renderer, set_output_transform(no_transformation))
         .InSequence(seq);
-    EXPECT_CALL(mock_renderer, set_rotation(mir_orientation_normal))
+    EXPECT_CALL(mock_renderer, set_viewport(screen))
         .InSequence(seq);
     EXPECT_CALL(mock_renderer, render(IsEmpty()))
         .InSequence(seq);
-    EXPECT_CALL(display_buffer, gl_swap_buffers())
-        .InSequence(seq);
-    EXPECT_CALL(display_buffer, flip())
-        .InSequence(seq);
 
-    EXPECT_CALL(display_buffer, post_renderables_if_optimizable(_))
+    EXPECT_CALL(display_buffer, overlay(_))
         .InSequence(seq)
         .WillOnce(Return(true));
     //we should be testing that post_buffer is called, not just that
     //we check the bits on the compositor buffer
-    EXPECT_CALL(display_buffer, post_renderables_if_optimizable(_))
+    EXPECT_CALL(display_buffer, overlay(_))
         .InSequence(seq)
         .WillOnce(Return(false));
-    EXPECT_CALL(display_buffer, make_current())
+    EXPECT_CALL(display_buffer, transformation())
         .InSequence(seq);
-    EXPECT_CALL(display_buffer, orientation())
+    EXPECT_CALL(mock_renderer, set_output_transform(no_transformation))
         .InSequence(seq);
-    EXPECT_CALL(mock_renderer, set_rotation(mir_orientation_normal))
+    EXPECT_CALL(mock_renderer, set_viewport(screen))
         .InSequence(seq);
     EXPECT_CALL(mock_renderer, render(IsEmpty()))
         .InSequence(seq);
-    EXPECT_CALL(display_buffer, gl_swap_buffers())
-        .InSequence(seq);
-    EXPECT_CALL(display_buffer, flip())
-        .InSequence(seq);
 
     mc::DefaultDisplayBufferCompositor compositor(
         display_buffer,
@@ -283,9 +293,9 @@ TEST_F(DefaultDisplayBufferCompositor, occluded_surfaces_are_not_rendered)
     using namespace testing;
     EXPECT_CALL(display_buffer, view_area())
         .WillRepeatedly(Return(screen));
-    EXPECT_CALL(display_buffer, orientation())
-        .WillOnce(Return(mir_orientation_normal));
-    EXPECT_CALL(display_buffer, post_renderables_if_optimizable(_))
+    EXPECT_CALL(display_buffer, transformation())
+        .WillOnce(Return(no_transformation));
+    EXPECT_CALL(display_buffer, overlay(_))
         .WillRepeatedly(Return(false));
 
     auto window0 = std::make_shared<mtd::FakeRenderable>(geom::Rectangle{{99,99},{2,2}});
@@ -295,15 +305,7 @@ TEST_F(DefaultDisplayBufferCompositor, occluded_surfaces_are_not_rendered)
 
     mg::RenderableList const visible{window0, window3};
 
-    Sequence seq;
-    EXPECT_CALL(display_buffer, make_current())
-        .InSequence(seq);
-    EXPECT_CALL(mock_renderer, render(ContainerEq(visible)))
-        .InSequence(seq);
-    EXPECT_CALL(display_buffer, gl_swap_buffers())
-        .InSequence(seq);
-    EXPECT_CALL(display_buffer, flip())
-        .InSequence(seq);
+    EXPECT_CALL(mock_renderer, render(ContainerEq(visible)));
 
     mc::DefaultDisplayBufferCompositor compositor(
         display_buffer,
@@ -323,14 +325,11 @@ struct MockSceneElement : mc::SceneElement
 {
     MockSceneElement(std::shared_ptr<mg::Renderable> const& renderable)
     {
-        ON_CALL(*this, is_a_surface())
-            .WillByDefault(testing::Return(true));
         ON_CALL(*this, renderable())
             .WillByDefault(testing::Return(renderable));
     }
 
     MOCK_CONST_METHOD0(renderable, std::shared_ptr<mir::graphics::Renderable>());
-    MOCK_CONST_METHOD0(is_a_surface, bool());
     MOCK_METHOD0(rendered, void());
     MOCK_METHOD0(occluded, void());
 };
