@@ -1,8 +1,8 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
 #include "mir/graphics/overlapping_output_grouping.h"
@@ -24,7 +22,6 @@
 #include <gtest/gtest.h>
 
 #include <vector>
-#include <algorithm>
 
 namespace mg = mir::graphics;
 namespace geom = mir::geometry;
@@ -60,18 +57,7 @@ public:
     {
     }
 
-    void for_each_card(std::function<void(mg::DisplayConfigurationCard const&)> f) const
-    {
-        mg::DisplayConfigurationCard card
-        {
-            mg::DisplayConfigurationCardId{1},
-            outputs.size()
-        };
-
-        f(card);
-    }
-
-    void for_each_output(std::function<void(mg::DisplayConfigurationOutput const&)> f) const
+    void for_each_output(std::function<void(mg::DisplayConfigurationOutput const&)> f) const override
     {
         uint32_t i = 1;
         for (auto const& info : outputs)
@@ -83,6 +69,7 @@ public:
             {
                 mg::DisplayConfigurationOutputId(i),
                 mg::DisplayConfigurationCardId{1},
+                mg::DisplayConfigurationLogicalGroupId{0},
                 mg::DisplayConfigurationOutputType::svideo,
                 {},
                 modes,
@@ -96,7 +83,12 @@ public:
                 info.power_mode,
                 info.orientation,
                 1.0f,
-                mir_form_factor_monitor
+                mir_form_factor_monitor,
+                mir_subpixel_arrangement_unknown,
+                {},
+                mir_output_gamma_unsupported,
+                {},
+                {}
             };
 
             f(output);
@@ -108,6 +100,11 @@ public:
     {
     }
 
+    std::unique_ptr<mg::DisplayConfiguration> clone() const override
+    {
+        return {};
+    }
+
     std::vector<OutputInfo> outputs;
 };
 
