@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,24 +12,26 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Robert Carr <robert.carr@canonical.com>
  */
 
 #include "mir/events/event_builders.h"
 #include "mir/events/event_private.h" // only needed to validate motion_up/down mapping
 
 #include <gtest/gtest.h>
+#include <gmock/gmock.h>
+
+#include <linux/input.h>
 
 namespace mev = mir::events;
+using namespace ::testing;
 
 namespace
 {
-struct InputEventBuilder : public testing::Test
+struct InputEventBuilder : public Test
 {
     MirInputDeviceId const device_id = 7;
     std::chrono::nanoseconds const timestamp = std::chrono::nanoseconds(39);
-    uint64_t const mac = 0;
+    std::vector<uint8_t> const cookie{};
     MirInputEventModifiers const modifiers = mir_input_event_modifier_meta;
 };
 }
@@ -37,11 +39,12 @@ struct InputEventBuilder : public testing::Test
 TEST_F(InputEventBuilder, makes_valid_key_event)
 {
     MirKeyboardAction const action = mir_keyboard_action_down;
-    xkb_keysym_t const key_code = 34;
+    xkb_keysym_t const keysym = 34;
     int const scan_code = 17;
 
-   auto ev = mev::make_event(device_id, timestamp,
-       mac, action, key_code, scan_code, modifiers);
+   auto ev = mev::make_key_event(
+       device_id, timestamp,
+       cookie, action, keysym, scan_code, modifiers);
    auto e = ev.get();
 
    EXPECT_EQ(mir_event_type_input, mir_event_get_type(e));
@@ -49,7 +52,7 @@ TEST_F(InputEventBuilder, makes_valid_key_event)
    EXPECT_EQ(mir_input_event_type_key, mir_input_event_get_type(ie));
    auto kev = mir_input_event_get_keyboard_event(ie);
    EXPECT_EQ(action, mir_keyboard_event_action(kev));
-   EXPECT_EQ(key_code, mir_keyboard_event_key_code(kev));
+   EXPECT_EQ(keysym, mir_keyboard_event_keysym(kev));
    EXPECT_EQ(scan_code, mir_keyboard_event_scan_code(kev));
    EXPECT_EQ(modifiers, mir_keyboard_event_modifiers(kev));
 }
@@ -65,10 +68,11 @@ TEST_F(InputEventBuilder, makes_valid_touch_event)
     float pressure_values[] = {3, 9, 14.6};
     float touch_major_values[] = {11, 9, 14};
     float touch_minor_values[] = {13, 3, 9.13};
-    float size_values[] = {4, 9, 6};
+    float size_values[] = {13, 9, 14};
 
-   auto ev = mev::make_event(device_id, timestamp,
-       mac, modifiers);
+   auto ev = mev::make_touch_event(
+       device_id, timestamp,
+       cookie, modifiers);
    for (unsigned i = 0; i < touch_count; i++)
    {
        mev::add_touch(*ev, touch_ids[i], actions[i], tooltypes[i], x_axis_values[i], y_axis_values[i],
@@ -104,7 +108,8 @@ TEST_F(InputEventBuilder, makes_valid_pointer_event)
     float x_axis_value = 3.9, y_axis_value = 7.4, hscroll_value = .9, vscroll_value = .3;
     auto const relative_x_value = 0.0;
     auto const relative_y_value = 0.0;
-    auto ev = mev::make_event(device_id, timestamp, mac, modifiers, 
+    auto ev = mev::make_pointer_event(
+        device_id, timestamp, cookie, modifiers,
         action, depressed_buttons, x_axis_value, y_axis_value,
         hscroll_value, vscroll_value, relative_x_value, relative_y_value);
     auto e = ev.get();
@@ -133,7 +138,7 @@ TEST_F(InputEventBuilder, maps_single_touch_down_to_motion_down)
 {
     MirTouchAction action =  mir_touch_action_down;
 
-    auto ev = mev::make_event(device_id, timestamp, mac, modifiers);
+    auto ev = mev::make_touch_event(device_id, timestamp, cookie, modifiers);
     mev::add_touch(*ev, 0, action, mir_touch_tooltype_finger, 0, 0, 0, 0, 0, 0);
     auto e = ev.get();
 
@@ -149,7 +154,7 @@ TEST_F(InputEventBuilder, maps_single_touch_up_to_motion_up)
 {
     MirTouchAction action =  mir_touch_action_up;
 
-    auto ev = mev::make_event(device_id, timestamp, mac, modifiers);
+    auto ev = mev::make_touch_event(device_id, timestamp, cookie, modifiers);
     mev::add_touch(*ev, 0, action, mir_touch_tooltype_finger, 0, 0, 0, 0, 0, 0);
     auto e = ev.get();
 
@@ -167,7 +172,8 @@ TEST_F(InputEventBuilder, map_to_hover_if_no_button_pressed)
     auto const relative_x_value = 0.0;
     auto const relative_y_value = 0.0;
     MirPointerAction action = mir_pointer_action_motion;
-    auto ev = mev::make_event(device_id, timestamp, mac, modifiers,
+    auto ev = mev::make_pointer_event(
+        device_id, timestamp, cookie, modifiers,
         action, 0, x_axis_value, y_axis_value,
         hscroll_value, vscroll_value, relative_x_value, relative_y_value);
     auto e = ev.get();
@@ -178,3 +184,45 @@ TEST_F(InputEventBuilder, map_to_hover_if_no_button_pressed)
     EXPECT_EQ(modifiers, mir_pointer_event_modifiers(pev));
     EXPECT_EQ(action, mir_pointer_event_action(pev));
 }
+
+TEST_F(InputEventBuilder, when_creating_input_device_state_event_it_has_supplied_properties)
+{
+    auto const pos_x = 12.1f;
+    auto const pos_y = 53.2f;
+    auto const button_state = mir_pointer_button_primary|mir_pointer_button_secondary;
+    auto const modifiers = mir_input_event_modifier_ctrl_right | mir_input_event_modifier_ctrl;
+    std::vector<uint32_t> const pressed_keys = {KEY_LEFTALT, KEY_M};
+
+    auto ev = mev::make_input_configure_event(
+        timestamp,
+        button_state,
+        modifiers,
+        pos_x,
+        pos_y,
+        {
+            mev::InputDeviceState{MirInputDeviceId{3}, pressed_keys, 0},
+            mev::InputDeviceState{MirInputDeviceId{2}, {}, button_state}});
+
+    EXPECT_THAT(mir_event_get_type(ev.get()), Eq(mir_event_type_input_device_state));
+    auto ids_event = mir_event_get_input_device_state_event(ev.get());
+
+    EXPECT_THAT(mir_input_device_state_event_time(ids_event), Eq(timestamp.count()));
+    EXPECT_THAT(mir_input_device_state_event_modifiers(ids_event), Eq(modifiers));
+    EXPECT_THAT(mir_input_device_state_event_pointer_axis(ids_event, mir_pointer_axis_x), Eq(pos_x));
+    EXPECT_THAT(mir_input_device_state_event_pointer_axis(ids_event, mir_pointer_axis_y), Eq(pos_y));
+    EXPECT_THAT(mir_input_device_state_event_pointer_buttons(ids_event), Eq(button_state));
+    EXPECT_THAT(mir_input_device_state_event_device_count(ids_event), Eq(2));
+
+    EXPECT_THAT(mir_input_device_state_event_device_id(ids_event, 0), Eq(MirInputDeviceId{3}));
+    auto const pressed_keys_count = mir_input_device_state_event_device_pressed_keys_count(ids_event, 0);
+    EXPECT_THAT(pressed_keys_count, Eq(2));
+    for (uint32_t i = 0; i < pressed_keys_count; i++)
+    {
+        EXPECT_THAT(mir_input_device_state_event_device_pressed_keys_for_index(ids_event, 0, i), Eq(pressed_keys[i]));
+    }
+    EXPECT_THAT(mir_input_device_state_event_device_pointer_buttons(ids_event, 0), Eq(0));
+
+    EXPECT_THAT(mir_input_device_state_event_device_id(ids_event, 1), Eq(MirInputDeviceId{2}));
+    EXPECT_THAT(mir_input_device_state_event_device_pressed_keys_count(ids_event, 1), Eq(0));
+    EXPECT_THAT(mir_input_device_state_event_device_pointer_buttons(ids_event, 1), Eq(button_state));
+}
