@@ -1,8 +1,8 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Authored by: Robert Carr <robert.carr@canonical.com>
  */
 
 #include "src/server/input/surface_input_dispatcher.h"
@@ -21,12 +19,14 @@
 #include "mir/events/event_builders.h"
 #include "mir/events/event_private.h"
 #include "mir/scene/observer.h"
+#include "mir/scene/surface_observer.h"
 #include "mir/thread_safe_list.h"
 
 #include "mir/test/event_matchers.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/doubles/stub_input_scene.h"
 #include "mir/test/doubles/mock_surface.h"
+#include "mir/test/doubles/explicit_executor.h"
 
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
@@ -70,10 +70,10 @@ struct StubInputScene : public mtd::StubInputScene
 {
     std::shared_ptr<mtd::MockSurface> add_surface(geom::Rectangle const& geometry)
     {
-        auto surface = std::make_shared<MockSurfaceWithGeometry>(geometry);
+        auto surface = std::make_shared<NiceMock<MockSurfaceWithGeometry>>(geometry);
         surfaces.add(surface);
 
-        observer->surface_added(surface.get());
+        observer->surface_added(surface);
         
         return surface;
     }
@@ -81,7 +81,7 @@ struct StubInputScene : public mtd::StubInputScene
     void remove_surface(std::shared_ptr<ms::Surface> const& surface)
     {
         surfaces.remove(surface);
-        observer->surface_removed(surface.get());
+        observer->surface_removed(surface);
     }
 
     std::shared_ptr<mtd::MockSurface> add_surface()
@@ -101,7 +101,7 @@ struct StubInputScene : public mtd::StubInputScene
         assert(observer == nullptr);
         observer = new_observer;
 	surfaces.for_each([this](std::shared_ptr<ms::Surface> const& surface) {
-		observer->surface_exists(surface.get());
+		observer->surface_exists(surface);
         });
     }
     
@@ -138,13 +138,15 @@ struct FakeKeyboard
     }
     mir::EventUPtr press(int scan_code = 7)
     {
-        return mev::make_event(id, std::chrono::nanoseconds(0), 0,
-	    mir_keyboard_action_down, 0, scan_code, mir_input_event_modifier_alt);
+        return mev::make_key_event(
+            id, std::chrono::nanoseconds(0), std::vector<uint8_t>{},
+            mir_keyboard_action_down, 0, scan_code, mir_input_event_modifier_alt);
     }
     mir::EventUPtr release(int scan_code = 7)
     {
-        return mev::make_event(id, std::chrono::nanoseconds(0), 0,
-	    mir_keyboard_action_up, 0, scan_code, mir_input_event_modifier_alt);
+        return mev::make_key_event(
+            id, std::chrono::nanoseconds(0), std::vector<uint8_t>{},
+            mir_keyboard_action_up, 0, scan_code, mir_input_event_modifier_alt);
     }
     MirInputDeviceId const id;
 };
@@ -159,7 +161,8 @@ struct FakePointer
 
     mir::EventUPtr move_to(geom::Point const& location)
     {
-        return mev::make_event(id, std::chrono::nanoseconds(0), 0,
+        return mev::make_pointer_event(
+            id, std::chrono::nanoseconds(0), std::vector<uint8_t>{},
             0, mir_pointer_action_motion, buttons,
             location.x.as_int(), location.y.as_int(),
             0, 0, 0, 0);
@@ -168,7 +171,8 @@ struct FakePointer
     {
         buttons &= ~button;
 
-        return mev::make_event(id, std::chrono::nanoseconds(0), 0,
+        return mev::make_pointer_event(
+            id, std::chrono::nanoseconds(0), std::vector<uint8_t>{},
             0, mir_pointer_action_button_up, buttons,
             location.x.as_int(), location.y.as_int(),
             0, 0, 0, 0);
@@ -177,7 +181,8 @@ struct FakePointer
     {
         buttons |= button;
         
-        return mev::make_event(id, std::chrono::nanoseconds(0), 0,
+        return mev::make_pointer_event(
+            id, std::chrono::nanoseconds(0), std::vector<uint8_t>{},
             0, mir_pointer_action_button_down, buttons,
             location.x.as_int(), location.y.as_int(),
             0, 0, 0, 0);
@@ -196,7 +201,7 @@ struct FakeToucher
     
     mir::EventUPtr move_to(geom::Point const& point)
     {
-        auto ev = mev::make_event(id, std::chrono::nanoseconds(0), 0, 0);
+        auto ev = mev::make_touch_event(id, std::chrono::nanoseconds(0), std::vector<uint8_t>{}, 0);
         mev::add_touch(*ev, 0, mir_touch_action_change,
                        mir_touch_tooltype_finger, point.x.as_int(), point.y.as_int(),
                        touched ? 1.0 : 0.0,
@@ -209,7 +214,7 @@ struct FakeToucher
     {
         touched = true;
         
-        auto ev = mev::make_event(id, std::chrono::nanoseconds(0), 0, 0);
+        auto ev = mev::make_touch_event(id, std::chrono::nanoseconds(0), std::vector<uint8_t>{}, 0);
         mev::add_touch(*ev, 0, mir_touch_action_down,
                        mir_touch_tooltype_finger, point.x.as_int(), point.y.as_int(),
                        1.0, 1.0, 1.0, 1.0);
@@ -219,7 +224,7 @@ struct FakeToucher
     {
         touched = true;
 
-        auto ev = mev::make_event(id, std::chrono::nanoseconds(0), 0, 0);
+        auto ev = mev::make_touch_event(id, std::chrono::nanoseconds(0), std::vector<uint8_t>{}, 0);
         mev::add_touch(*ev, 0, mir_touch_action_down,
                        mir_touch_tooltype_finger, point1.x.as_int(), point1.y.as_int(),
                        1.0, 1.0, 1.0, 1.0);
@@ -232,7 +237,7 @@ struct FakeToucher
     {
         touched = false;
         
-        auto ev = mev::make_event(id, std::chrono::nanoseconds(0), 0, 0);
+        auto ev = mev::make_touch_event(id, std::chrono::nanoseconds(0), std::vector<uint8_t>{}, 0);
         mev::add_touch(*ev, 0, mir_touch_action_up,
                        mir_touch_tooltype_finger, point.x.as_int(), point.y.as_int(),
                        0.0, 0.0, 0.0, 0.0);
@@ -242,7 +247,7 @@ struct FakeToucher
     {
         touched = false;
 
-        auto ev = mev::make_event(id, std::chrono::nanoseconds(0), 0, 0);
+        auto ev = mev::make_touch_event(id, std::chrono::nanoseconds(0), std::vector<uint8_t>{}, 0);
         mev::add_touch(*ev, 0, mir_touch_action_up,
                        mir_touch_tooltype_finger, point1.x.as_int(), point1.y.as_int(),
                        1.0, 1.0, 1.0, 1.0);
@@ -255,124 +260,75 @@ struct FakeToucher
     MirInputDeviceId const id;
 };
 
+struct MockKeyboardObserver: mi::KeyboardObserver
+{
+    MOCK_METHOD1(keyboard_event, void(std::shared_ptr<MirEvent const> const& event));
+    MOCK_METHOD1(keyboard_focus_set, void(std::shared_ptr<mi::Surface> const& surface));
+};
+
 }
 
-TEST_F(SurfaceInputDispatcher, key_event_delivered_to_focused_surface)
+TEST_F(SurfaceInputDispatcher, key_event_not_delivered_to_surface)
 {
     auto surface = scene.add_surface();
 
     FakeKeyboard keyboard;
     auto event = keyboard.press();
 
-    EXPECT_CALL(*surface, consume(mt::MirKeyEventMatches(*event))).Times(1);
-
-    dispatcher.start();
-
-    dispatcher.set_focus(surface);
-    EXPECT_TRUE(dispatcher.dispatch(*event));
-}
-
-TEST_F(SurfaceInputDispatcher, key_event_dropped_if_no_surface_focused)
-{
-    auto surface = scene.add_surface();
-    
-    EXPECT_CALL(*surface, consume(_)).Times(0);
-
-    dispatcher.start();
-
-    FakeKeyboard keyboard;
-    EXPECT_FALSE(dispatcher.dispatch(*keyboard.press()));
-}
-
-TEST_F(SurfaceInputDispatcher, inconsistent_key_events_dropped)
-{
-    auto surface = scene.add_surface();
-
     EXPECT_CALL(*surface, consume(_)).Times(0);
 
     dispatcher.start();
 
     dispatcher.set_focus(surface);
-
-    FakeKeyboard keyboard;
-    EXPECT_FALSE(dispatcher.dispatch(*keyboard.release()));
+    dispatcher.dispatch(std::move(event));
 }
 
-TEST_F(SurfaceInputDispatcher, key_state_is_consistent_per_client)
-{
-    auto surface_1 = scene.add_surface();
-    auto surface_2 = scene.add_surface();
-
-    FakeKeyboard keyboard;
-    auto down_event = keyboard.press();
-    auto up_event = keyboard.release();
-
-    EXPECT_CALL(*surface_1, consume(mt::MirKeyEventMatches(*down_event))).Times(1);
-    EXPECT_CALL(*surface_2, consume(_)).Times(0);
-
-    dispatcher.start();
-
-    dispatcher.set_focus(surface_1);
-    EXPECT_TRUE(dispatcher.dispatch(*down_event));
-    dispatcher.set_focus(surface_2);
-    EXPECT_FALSE(dispatcher.dispatch(*up_event));
-}
-
-TEST_F(SurfaceInputDispatcher, inconsistent_key_down_dropped)
+TEST_F(SurfaceInputDispatcher, key_event_delivered_to_keyboard_observer)
 {
     auto surface = scene.add_surface();
+    mtd::ExplicitExecutor executor;
 
     FakeKeyboard keyboard;
     auto event = keyboard.press();
 
-    InSequence seq;
-    EXPECT_CALL(*surface, consume(mt::MirKeyEventMatches(*event))).Times(1);
+    auto const kb_observer = std::make_shared<MockKeyboardObserver>();
+    EXPECT_CALL(*kb_observer, keyboard_event(mt::MirKeyboardEventMatches(event.get())));
 
     dispatcher.start();
-
-    dispatcher.set_focus(surface);
-    EXPECT_TRUE(dispatcher.dispatch(*event));
-    EXPECT_FALSE(dispatcher.dispatch(*event));
-    EXPECT_FALSE(dispatcher.dispatch(*event));
+    dispatcher.register_interest(kb_observer, executor);
+    dispatcher.dispatch(std::move(event));
+    executor.execute();
 }
 
-TEST_F(SurfaceInputDispatcher, device_reset_resets_key_state_consistency)
+TEST_F(SurfaceInputDispatcher, keyboard_focus_delivered_to_keyboard_observer)
 {
     auto surface = scene.add_surface();
+    mtd::ExplicitExecutor executor;
 
-    auto device_id = MirInputDeviceId{1};
-    FakeKeyboard keyboard(device_id);
-    auto down_event = keyboard.press(11);
-    auto release_event = keyboard.release(11);
+    auto const kb_observer = std::make_shared<MockKeyboardObserver>();
+    EXPECT_CALL(*kb_observer, keyboard_focus_set(Eq(surface)));
 
     dispatcher.start();
-
+    dispatcher.register_interest(kb_observer, executor);
     dispatcher.set_focus(surface);
-    EXPECT_TRUE(dispatcher.dispatch(*down_event));
-    EXPECT_TRUE(dispatcher.dispatch(
-        *mev::make_event(mir_input_configuration_action_device_reset, device_id, std::chrono::nanoseconds{1})));
-    EXPECT_FALSE(dispatcher.dispatch(*release_event));
+    executor.execute();
 }
 
-TEST_F(SurfaceInputDispatcher, key_input_target_may_disappear_and_things_remain_quote_a_unquote_ok)
+TEST_F(SurfaceInputDispatcher, keyboard_focus_clear_delivered_to_keyboard_observer)
 {
-    auto surface_2 = scene.add_surface();
-    auto surface_1 = scene.add_surface();
+    auto surface = scene.add_surface();
+    mtd::ExplicitExecutor executor;
 
-    EXPECT_CALL(*surface_1, consume(_)).Times(AnyNumber());
-    EXPECT_CALL(*surface_2, consume(_)).Times(AnyNumber());
-
-    FakeKeyboard k;
-    auto an_ev = k.press(11);
-    auto another_ev = k.press(12);
+    auto const kb_observer = std::make_shared<NiceMock<MockKeyboardObserver>>();
 
     dispatcher.start();
-    dispatcher.set_focus(surface_1);
-    EXPECT_TRUE(dispatcher.dispatch(*an_ev));
-    scene.remove_surface(surface_1);
-    EXPECT_FALSE(dispatcher.dispatch(*another_ev));
-    dispatcher.set_focus(surface_2);
-    EXPECT_TRUE(dispatcher.dispatch(*another_ev));
+    dispatcher.register_interest(kb_observer, executor);
+    dispatcher.set_focus(surface);
+    executor.execute();
+
+    EXPECT_CALL(*kb_observer, keyboard_focus_set(Eq(nullptr)));
+    dispatcher.set_focus(nullptr);
+    executor.execute();
 }
 
 TEST_F(SurfaceInputDispatcher, pointer_motion_delivered_to_client_under_pointer)
@@ -385,13 +341,12 @@ TEST_F(SurfaceInputDispatcher, pointer_motion_delivered_to_client_under_pointer)
 
     InSequence seq;
     EXPECT_CALL(*surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*surface, consume(mt::PointerEventWithPosition(1, 0))).Times(1);
     EXPECT_CALL(*surface, consume(mt::PointerLeaveEvent())).Times(1);
 
     dispatcher.start();
 
-    EXPECT_TRUE(dispatcher.dispatch(*pointer.move_to({1, 0})));
-    EXPECT_TRUE(dispatcher.dispatch(*pointer.move_to({5, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(pointer.move_to({1, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(pointer.move_to({5, 0})));
 }
 
 TEST_F(SurfaceInputDispatcher, pointer_delivered_only_to_top_surface)
@@ -403,15 +358,13 @@ TEST_F(SurfaceInputDispatcher, pointer_delivered_only_to_top_surface)
 
     InSequence seq;
     EXPECT_CALL(*top_surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*top_surface, consume(mt::PointerEventWithPosition(1, 0))).Times(1);
     EXPECT_CALL(*surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*surface, consume(mt::PointerEventWithPosition(1, 0))).Times(1);
-    
+
     dispatcher.start();
 
-    EXPECT_TRUE(dispatcher.dispatch(*pointer.move_to({1, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(pointer.move_to({1, 0})));
     scene.remove_surface(top_surface);
-    EXPECT_TRUE(dispatcher.dispatch(*pointer.move_to({1, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(pointer.move_to({1, 0})));
 }
 
 TEST_F(SurfaceInputDispatcher, pointer_may_move_between_adjacent_surfaces)
@@ -425,17 +378,15 @@ TEST_F(SurfaceInputDispatcher, pointer_may_move_between_adjacent_surfaces)
 
     InSequence seq;
     EXPECT_CALL(*surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*surface, consume(mt::PointerEventWithPosition(1, 1))).Times(1);
     EXPECT_CALL(*surface, consume(mt::PointerLeaveEvent())).Times(1);
     EXPECT_CALL(*another_surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*another_surface, consume(mt::PointerEventWithPosition(1, 1))).Times(1);
     EXPECT_CALL(*another_surface, consume(mt::PointerLeaveEvent())).Times(1);
-    
+
     dispatcher.start();
 
-    EXPECT_TRUE(dispatcher.dispatch(*pointer.move_to({1, 1})));
-    EXPECT_TRUE(dispatcher.dispatch(*pointer.move_to({6, 6})));
-    EXPECT_TRUE(dispatcher.dispatch(*pointer.move_to({11, 11})));
+    EXPECT_TRUE(dispatcher.dispatch(pointer.move_to({1, 1})));
+    EXPECT_TRUE(dispatcher.dispatch(pointer.move_to({6, 6})));
+    EXPECT_TRUE(dispatcher.dispatch(pointer.move_to({11, 11})));
 }
 
 // We test that a client will receive pointer events following a button down
@@ -460,33 +411,9 @@ TEST_F(SurfaceInputDispatcher, gestures_persist_over_button_down)
     
     dispatcher.start();
 
-    EXPECT_TRUE(dispatcher.dispatch(*ev_1));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_2));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_3));
-}
-
-TEST_F(SurfaceInputDispatcher, gestures_terminated_by_device_reset)
-{
-    auto surface = scene.add_surface({{0, 0}, {5, 5}});
-    auto another_surface = scene.add_surface({{5, 5}, {5, 5}});
-
-    MirInputDeviceId device_id{1};
-    FakePointer pointer(device_id);
-    auto ev_1 = pointer.press_button({0, 0});
-    auto ev_2 = pointer.move_to({6, 6});
-
-    InSequence seq;
-    EXPECT_CALL(*surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*surface, consume(mt::ButtonDownEvent(0,0))).Times(1);
-    EXPECT_CALL(*another_surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*another_surface, consume(mt::PointerEventWithPosition(1, 1))).Times(1);
-    
-    dispatcher.start();
-
-    EXPECT_TRUE(dispatcher.dispatch(*ev_1));
-    EXPECT_TRUE(dispatcher.dispatch(
-        *mev::make_event(mir_input_configuration_action_device_reset, device_id, std::chrono::nanoseconds{1})));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_2));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_1)));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_2)));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_3)));
 }
 
 TEST_F(SurfaceInputDispatcher, pointer_gestures_may_transfer_over_buttons)
@@ -513,11 +440,11 @@ TEST_F(SurfaceInputDispatcher, pointer_gestures_may_transfer_over_buttons)
 
     dispatcher.start();
 
-    EXPECT_TRUE(dispatcher.dispatch(*ev_1));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_2));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_3));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_4));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_5));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_1)));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_2)));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_3)));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_4)));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_5)));
 }
 
 TEST_F(SurfaceInputDispatcher, pointer_gesture_target_may_vanish_and_the_situation_remains_hunky_dorey)
@@ -534,14 +461,13 @@ TEST_F(SurfaceInputDispatcher, pointer_gesture_target_may_vanish_and_the_situati
     EXPECT_CALL(*surface, consume(mt::PointerEnterEvent())).Times(1);
     EXPECT_CALL(*surface, consume(mt::ButtonDownEvent(0,0))).Times(1);
     EXPECT_CALL(*another_surface, consume(mt::PointerEnterEvent())).Times(1);
-    EXPECT_CALL(*another_surface, consume(mt::PointerEventWithPosition(1,1))).Times(1);
-    
+
     dispatcher.start();
 
-    EXPECT_TRUE(dispatcher.dispatch(*ev_1));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_1)));
     scene.remove_surface(surface);
-    EXPECT_FALSE(dispatcher.dispatch(*ev_2));
-    EXPECT_TRUE(dispatcher.dispatch(*ev_3));
+    EXPECT_FALSE(dispatcher.dispatch(std::move(ev_2)));
+    EXPECT_TRUE(dispatcher.dispatch(std::move(ev_3)));
 }
 
 TEST_F(SurfaceInputDispatcher, touch_delivered_to_surface)
@@ -555,8 +481,8 @@ TEST_F(SurfaceInputDispatcher, touch_delivered_to_surface)
     dispatcher.start();
 
     FakeToucher toucher;
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({1,1})));
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.release_at({1,1})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({1,1})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.release_at({1,1})));
 }
 
 TEST_F(SurfaceInputDispatcher, touch_delivered_only_to_top_surface)
@@ -573,8 +499,8 @@ TEST_F(SurfaceInputDispatcher, touch_delivered_only_to_top_surface)
     dispatcher.start();
 
     FakeToucher toucher;
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({1,1})));
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.release_at({2,2})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({1,1})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.release_at({2,2})));
 }
 
 TEST_F(SurfaceInputDispatcher, gestures_persist_over_touch_down)
@@ -591,9 +517,9 @@ TEST_F(SurfaceInputDispatcher, gestures_persist_over_touch_down)
     dispatcher.start();
     
     FakeToucher toucher;
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({0, 0})));
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.move_to({2, 2})));
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.release_at({2, 2})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({0, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.move_to({2, 2})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.release_at({2, 2})));
 }
 
 TEST_F(SurfaceInputDispatcher, touch_target_switches_on_finger_down)
@@ -611,9 +537,9 @@ TEST_F(SurfaceInputDispatcher, touch_target_switches_on_finger_down)
     dispatcher.start();
     
     FakeToucher toucher;
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({0, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({0, 0})));
     // Note: No touch release event produced
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({5, 5})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({5, 5})));
 }
 
 TEST_F(SurfaceInputDispatcher, touch_target_switches_on_fingers_down)
@@ -630,9 +556,9 @@ TEST_F(SurfaceInputDispatcher, touch_target_switches_on_fingers_down)
     dispatcher.start();
 
     FakeToucher toucher;
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({0, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({0, 0})));
     // Note: No touch release event produced
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touches_at({5, 5}, {6, 6})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touches_at({5, 5}, {6, 6})));
 }
 
 TEST_F(SurfaceInputDispatcher, touch_gestures_terminated_by_release_all_touches)
@@ -646,29 +572,9 @@ TEST_F(SurfaceInputDispatcher, touch_gestures_terminated_by_release_all_touches)
     dispatcher.start();
 
     FakeToucher toucher;
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touches_at({5, 5}, {6, 6})));
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.releases_at({5, 5}, {6, 6})));
-    EXPECT_FALSE(dispatcher.dispatch(*toucher.move_to({5, 6})));
-}
-
-TEST_F(SurfaceInputDispatcher, touch_gestures_terminated_by_device_reset)
-{
-    auto left_surface = scene.add_surface({{0, 0}, {1, 1}});
-    auto right_surface = scene.add_surface({{1, 1}, {1, 1}});
-
-    MirInputDeviceId device_id{1};
-    FakeToucher toucher(device_id);
-
-    InSequence seq;
-    EXPECT_CALL(*left_surface, consume(mt::TouchEvent(0, 0))).Times(1);
-    EXPECT_CALL(*right_surface, consume(mt::TouchEvent(0, 0))).Times(1);
-
-    dispatcher.start();
-    
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({0, 0})));
-    EXPECT_TRUE(dispatcher.dispatch(
-        *mev::make_event(mir_input_configuration_action_device_reset, device_id, std::chrono::nanoseconds{1})));
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({1, 1})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touches_at({5, 5}, {6, 6})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.releases_at({5, 5}, {6, 6})));
+    EXPECT_FALSE(dispatcher.dispatch(toucher.move_to({5, 6})));
 }
 
 TEST_F(SurfaceInputDispatcher, touch_gesture_target_may_vanish_but_things_continue_to_function_as_intended)
@@ -683,8 +589,8 @@ TEST_F(SurfaceInputDispatcher, touch_gesture_target_may_vanish_but_things_contin
     dispatcher.start();
     
     FakeToucher toucher;
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({0, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({0, 0})));
     scene.remove_surface(surface_2);
-    EXPECT_FALSE(dispatcher.dispatch(*toucher.release_at({0, 0})));
-    EXPECT_TRUE(dispatcher.dispatch(*toucher.touch_at({0, 0})));
+    EXPECT_FALSE(dispatcher.dispatch(toucher.release_at({0, 0})));
+    EXPECT_TRUE(dispatcher.dispatch(toucher.touch_at({0, 0})));
 }
